@@ -40,7 +40,8 @@
 #define BACKLOG 1024     // how many pending connections queue will hold
 #define MAX_BUFF 32768  //read buffer size. bigger faster, but memory foot print bigger
 #define MAX_RETRY 9   //number of connection retry
-#define VERSION "0.5"
+#define VERSION "0.6"
+#define MAX_TIMERS 10 //maximum number of running timers
 
 /*
 Structure we use for each client's connection. 
@@ -63,12 +64,18 @@ struct client {
         PyObject *wsgi_cb;
         int response_iter_sent; //-2: nothing sent, -1: header sent, 0-9999: iter sent
         char *response_header;
-	int response_header_length;
+        int response_header_length;
         PyObject *response_content;
         PyObject *response_content_obj;
         FILE *response_fp; // file of the sent file
 };
 
+struct TimerObj {
+        ev_timer timerwatcher;
+        float delay;
+        PyObject *py_cb;
+};
+
 
 /*
 Somme  global variables
@@ -82,7 +89,8 @@ PyObject *py_base_module;  //to store the fapws.base python module
 PyObject *py_config_module; //to store the fapws.config module
 PyObject *py_registered_uri; //list containing the uri registered and their associated wsgi callback.
 PyObject *py_generic_cb=NULL; 
-
+struct TimerObj *list_timers[MAX_TIMERS];
+int list_timers_i=0; //number of values entered in the array list_timers
 
 
 /*
@@ -1104,8 +1112,8 @@ http://beej.us/guide/bgnet/output/html/singlepage/bgnet.html
 
     // loop through all the results and bind to the first we can
     for(p = servinfo; p != NULL; p = p->ai_next) {
-        if ((sockfd = socket(p->ai_family, p->ai_socktype,
-                p->ai_protocol)) == -1) {
+        sockfd = socket(p->ai_family, p->ai_socktype, p->ai_protocol);
+        if (sockfd == -1) {
             perror("server: socket");
             continue;
         }
@@ -1140,6 +1148,28 @@ http://beej.us/guide/bgnet/output/html/singlepage/bgnet.html
     return Py_None;
 }
 
+static void 
+timer_cb(struct ev_loop *loop, ev_timer *w, int revents)
+{
+    struct TimerObj *timer= ((struct TimerObj*) (((char*)w) - offsetof(struct TimerObj,timerwatcher)));
+    PyObject *resp = PyEval_CallObject(timer->py_cb, NULL);
+    if (resp==NULL)
+    {
+        if (PyErr_Occurred()) 
+        { 
+             PyErr_Print();
+        }
+        ev_timer_stop(loop, w);
+    }
+    if (resp==Py_False)
+    {
+        ev_timer_stop(loop, w);
+    }
+    Py_DECREF(resp);
+}
+
+
+
 /*
 Procedure exposed in Python will generate and start the event loop
 */
@@ -1147,8 +1177,10 @@ static PyObject *
 py_run_loop(PyObject *self, PyObject *args)
 {
     char *backend="";
+    int i;
     ev_io accept_watcher;
     ev_signal signal_watcher, signal_watcher2;
+    struct TimerObj *timer;
     struct ev_loop *loop = ev_default_loop (0);
     switch (ev_backend(loop))
     {
@@ -1172,20 +1204,19 @@ py_run_loop(PyObject *self, PyObject *args)
     ev_signal_start(loop, &signal_watcher);
     ev_signal_init(&signal_watcher2, sigpipe_cb, SIGPIPE);
     ev_signal_start(loop, &signal_watcher2);
+    if (list_timers_i>=0)
+    {
+        for (i=0; i<list_timers_i; i++)
+        {
+            timer=list_timers[i];
+            ev_timer_init(&timer->timerwatcher, timer_cb, timer->delay, timer->delay);
+            ev_timer_start(loop, &timer->timerwatcher);
+        }
+    }
     ev_loop (loop, 0);
     return Py_None;
 }
 
-/*
-Procedure exposed in Python to provide extension's version
-*/
-static PyObject *
-py_version(PyObject *self, PyObject *args)
-{
-    PyObject *pyres=Py_BuildValue("s", "3.b.0");
-    return pyres;
-}
-
 /*
 Procedure exposed in Python to provide libev's ABI version
 */
@@ -1266,10 +1297,72 @@ py_get_debug(PyObject *self, PyObject *args)
     return Py_BuildValue("i", debug);
 }
 
+/*
+Procedure exposed in Python to add a timer: delay, python callback method
+*/
+static PyObject *
+py_add_timer_cb(PyObject *self, PyObject *args)
+{
+    struct TimerObj *timer;
+    if (list_timers_i<MAX_TIMERS)
+    {
+        timer = calloc(1,sizeof(struct TimerObj));
+        if (!PyArg_ParseTuple(args, "fO", &timer->delay, &timer->py_cb)) 
+            return NULL;
+        list_timers[list_timers_i]=timer;
+        list_timers_i++;
+    } 
+    else
+    {
+        printf("Limit of maximum %i timers has been reached\n", list_timers_i);
+    }
+    
+    return PyInt_FromLong(list_timers_i);    
+}
+
+/*
+Procedure exposed in Python to stop a running timer: i
+*/
+static PyObject *
+py_stop_timer(PyObject *self, PyObject *args)
+{
+    int i;
+    struct TimerObj *timer;
+    struct ev_loop *loop = ev_default_loop(0);
+    if (!PyArg_ParseTuple(args, "i", &i)) 
+        return NULL;
+    timer=list_timers[i];
+    ev_timer_stop(loop, &timer->timerwatcher);
+    
+    return Py_None;    
+}
+
+/*
+Procedure exposed in Python to restart a running timer: i
+*/
+static PyObject *
+py_restart_timer(PyObject *self, PyObject *args)
+{
+    int i;
+    struct TimerObj *timer;
+    struct ev_loop *loop = ev_default_loop(0);
+    if (!PyArg_ParseTuple(args, "i", &i)) 
+        return NULL;
+    if (i<=list_timers_i)
+    {
+        timer=list_timers[i];
+        ev_timer_again(loop, &timer->timerwatcher);
+    }
+    else
+    {
+        printf("index out of range\n");
+    }
+    return Py_None;    
+}
+
 
 static PyMethodDef EvhttpMethods[] = {
     {"start", py_ev_start, METH_VARARGS, "Define evhttp sockets"},
-    {"version", py_version, METH_VARARGS, "return the version of the shared object"}, 
     {"set_base_module", py_set_base_module, METH_VARARGS, "set you base module"},
     {"run", py_run_loop, METH_VARARGS, "Run the main loop"},
     {"wsgi_cb", py_add_wsgi_cb, METH_VARARGS, "Add an uri and his wsgi callback in the list of uri to watch"},
@@ -1278,6 +1371,9 @@ static PyMethodDef EvhttpMethods[] = {
     {"set_debug", py_set_debug, METH_VARARGS, "Set the debug level"},
     {"get_debug", py_get_debug, METH_VARARGS, "Get the debug level"},
     {"libev_version", py_libev_version, METH_VARARGS, "Get the libev's ABI version you are using"},
+    {"add_timer", py_add_timer_cb, METH_VARARGS, "Add a timer"},
+    {"stop_timer", py_stop_timer, METH_VARARGS, "Stop a running timer"},
+    {"restart_timer", py_restart_timer, METH_VARARGS, "Restart an existing timer"},
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
